/*
 * FILE NAME
 *	arch/mips/vr41xx/vr4111/common/bcu.c
 *
 * BRIEF MODULE DESCRIPTION
 *	Bus Control Unit routines for the NEC VR4111.
 *
 * Author: Yoichi Yuasa
 *         yyuasa@mvista.com or source@mvista.com
 *
 * Copyright 2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * Changes:
 *  MontaVista Software Inc. <yyuasa@mvista.com> or <source@mvista.com>
 *  - New creation, NEC VR4122 and VR4131 are supported.
 *
 *  Yoichi Yuasa <yuasa@hh.iij4u.or.jp>  Tue,  5 Mar 2002
 *  -  Modified this code for VR4111 support.
 */
#include <linux/types.h>

#include <asm/io.h>
#include <asm/param.h>
#include <asm/time.h>

#define CLKSPEEDREG			KSEG1ADDR(0x0b000014)
#define calc_pclock(clksp)		((18432000 * 64) / ((clksp) & 0x001f))

static inline unsigned long calc_tclock(u16 clksp, unsigned long pclock)
{
	u16 div;

	if (!(clksp & 0x8000))
		div = 2;
	else if (!(clksp & 0x4000))
		div = 3;
	else if (!(clksp & 0x2000))
		div = 4;

	return pclock / div;
}

void vr41xx_bcu_init(void)
{
	unsigned long pclock, tclock;
	u16 clksp;

	clksp = readw(CLKSPEEDREG);
	pclock = calc_pclock(clksp);
	printk(KERN_INFO "PClock: %ldHz\n", pclock);

	tclock = calc_tclock(clksp, pclock);
	printk(KERN_INFO "TClock: %ldHz\n", tclock);

	mips_counter_frequency = tclock / 4;
}
