/*
 * FILE NAME
 *	arch/mips/vr41xx/vr4122/common/cmu.c
 *
 * BRIEF MODULE DESCRIPTION
 *	Clock Mask Unit routines for the NEC VR4122 and VR4131.
 *
 * Author: Yoichi Yuasa
 *         yyuasa@mvista.com or source@mvista.com
 *
 * Copyright 2001,2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * Changes:
 *  MontaVista Software Inc. <yyuasa@mvista.com> or <source@mvista.com>
 *  - New creation, NEC VR4122 and VR4131 are supported.
 */
#include <linux/spinlock.h>
#include <linux/types.h>

#include <asm/io.h>

#define CMUCLKMSK	KSEG1ADDR(0x0f000060)

rwlock_t vr41xx_cmu_lock = RW_LOCK_UNLOCKED;

void vr41xx_clock_supply(u16 mask)
{
	unsigned long flags;
	u16 val;

	write_lock_irqsave(&vr41xx_cmu_lock, flags);
	val = readw(CMUCLKMSK);
	val |= mask;
	writew(val, CMUCLKMSK);
	write_unlock_irqrestore(&vr41xx_cmu_lock, flags);
}

void vr41xx_clock_mask(u16 mask)
{
	unsigned long flags;
	u16 val;

	write_lock_irqsave(&vr41xx_cmu_lock, flags);
	val = readw(CMUCLKMSK);
	val &= ~mask;
	writew(val, CMUCLKMSK);
	write_unlock_irqrestore(&vr41xx_cmu_lock, flags);
}
